// (C) 2017-2020 by folkert van heusden, released under AGPL v3.0
#include <stdlib.h>
#include <string.h>

#include "resize.h"
#include "log.h"
#include "utils.h"

void picture_in_picture(resize *const r, uint8_t *const tgt, const int tgt_w, const int tgt_h, const uint8_t *const in, const int win, const int hin, const int perc, const pos_t pos)
{
	int wout = win * perc / 100, hwout = wout / 2;
	int hout = hin * perc / 100, hhout = hout / 2;

	uint8_t *temp = nullptr;

	r->do_resize(win, hin, in, wout, hout, &temp);

	const int qw = tgt_w / 4, qh = tgt_h / 4;
	int tx = 0, ty = 0;

	if (pos.type == upper_left) {
		tx = qw * 1 - hwout * 3 / 2;
		ty = qh - hhout * 3 / 2;
	}
	else if (pos.type == upper_center) {
		tx = qw * 2 - hwout;
		ty = qh - hhout * 3 / 2;
	}
	else if (pos.type == upper_right) {
		tx = qw * 3 - hwout * 2 / 3;
		ty = qh - hhout * 3 / 2;
	}
	else if (pos.type == center_left) {
		tx = qw * 1 - hwout * 3 / 2;
		ty = qh * 2 - hhout;
	}
	else if (pos.type == center_center) {
		tx = qw * 2 - hwout;
		ty = qh * 2 - hhout;
	}
	else if (pos.type == center_right) {
		tx = qw * 3 - hwout * 2 / 3;
		ty = qh * 2 - hhout;
	}
	else if (pos.type == lower_left) {
		tx = qw * 1 - hwout * 3 / 2;
		ty = qh * 3 - hhout * 2 / 3;
	}
	else if (pos.type == lower_center) {
		tx = qw * 2 - hwout;
		ty = qh * 3 - hhout * 2 / 3;
	}
	else if (pos.type == lower_right) {
		tx = qw * 3 - hwout * 2 / 3;
		ty = qh * 3 - hhout * 2 / 3;
	}
	else if (pos.type == xy) {
		tx = pos.x;
		ty = pos.y;
	}

	if (pos.type != none) {
		int width = std::min(wout, tgt_w - wout);

		if (width > 0) {
			for(int y=0; y<hout; y++) {
				int yo = y + ty;
				if (yo >= tgt_h)
					break;

				memcpy(&tgt[yo * tgt_w * 3 + tx * 3], &temp[y * wout * 3], width * 3);
			}
		}
	}

	free(temp);
}

resize::resize()
{
	log(LL_INFO, "Generic resizer instantiated");
}

resize::~resize()
{
}

void resize::do_resize(const int win, const int hin, const uint8_t *const in, const int wout, const int hout, uint8_t **out)
{
	*out = (uint8_t *)allocate_aligned(wout * hout * 3);

	const double maxw = std::max(win, wout);
	const double maxh = std::max(hin, hout);

	const double hins = hin / maxh;
	const double wins = win / maxw;
	const double houts = hout / maxh;
	const double wouts = wout / maxw;

	for(int y=0; y<maxh; y++) {
		const int in_scaled_y = y * hins;
		const int in_scaled_o = in_scaled_y * win * 3;
		const int out_scaled_y = y * houts;
		const int out_scaled_o = out_scaled_y * wout * 3;

		for(int x=0; x<maxw; x++) {
			int ino = in_scaled_o + int(x * wins) * 3;
			int outo = out_scaled_o + int(x * wouts) * 3;

			(*out)[outo + 0] = in[ino + 0];
			(*out)[outo + 1] = in[ino + 1];
			(*out)[outo + 2] = in[ino + 2];
		}
	}
}
